#include "UIMgr.h"

CUIMgr::CUIMgr()
{
	setlocale(LC_ALL, "");
	initscr();
	if (has_colors())
	{
		m_bHasColor = true;
		//		start_color();
	}
	else
	{
		m_bHasColor = false;
	}
	cbreak();
	noecho();
	curs_set(0);

	m_bRun = false;
	m_bPainLock = false;
	m_bPlaying = false;
	m_bPluginLock = false;
	m_bModeLock = false;
	m_nCurSel = 0;
	m_tSTime = m_tRTime = time(NULL);

	Layout();
}

CUIMgr::~CUIMgr()
{
	free_menu(m_modemenu);
	for (int i = 0; i < 5; ++i)
	{
		free_item(m_modeitems[i]);
	}
	free(m_modeitems);
	DestroyListMenu();
	endwin();
}

int CUIMgr::OnMsg(const char* lpszMsg, WPARAM wParam, LPARAM lParam)
{
	if (0 == strcmp(lpszMsg, MSG_NAME))
	{
		switch (wParam)
		{
		case 0://name
		{
			m_strName = (const char*)lParam;
			mvwhline(m_wins[1],1, 1, ' ', STD_WID-2);
			mvwprintw(m_wins[1], 1, 1, m_strName.c_str());
			wrefresh(m_wins[1]);
			// as loadfile send this msg,when recv,means playing ...
			m_bPlaying = true;
			m_fTPos = 0.0f;
			m_fTLen = 0.0f;
			m_strInfo = "";
		}
			break;
		case 1://samples
		case 2://rate
		{
			std::string strTmp = (const char*)lParam;
			m_strInfo += ("[" + strTmp + "]");
			mvwhline(m_wins[1],2, 1, ' ', STD_WID-2);
			mvwprintw(m_wins[1], 2, 1, m_strInfo.c_str());
			wrefresh(m_wins[1]);
		}
			break;
		case 3://tlen
		{
			if (m_fTLen >= 0.0f)
			{
				float* pfLen = (float*)lParam;
				m_fTLen = *pfLen;
			}
		}
			break;
		default:
			break;
		}

	}
	else if (0 == strcmp(lpszMsg, MSG_TPOS))
	{
		m_tRTime = time(NULL);
		float* pfPos = (float*)lParam;
		m_fTPos = *pfPos;
	}
	return 0;
}

void CUIMgr::Layout()
{
	m_menu = NULL;
	m_nLINES = LINES;
	m_nCOLS = COLS;

	m_wins[0] = newwin(m_nLINES, m_nCOLS, 0, 0);
	m_wins[1] = newwin(STD_HEG, STD_WID, 1, 1);
	m_wins[2] = newwin(m_nLINES-STD_HEG-2, STD_WID, STD_HEG+1, 1);
	m_wins[3] = newwin(m_nLINES-2, m_nCOLS-STD_WID-2, 1, STD_WID+1);

	for (int i = 0; i < 4; ++i)
	{
		if (isText)
		{
			wborder(m_wins[i],'|','|','-','-','+','+','+','+');
		}
		else
		{
			box(m_wins[i], 0, 0);				
		}
	}

	//play control
	mvwhline(m_wins[1],3, 2, '-', 21);
	mvwaddch(m_wins[1],3, 1, '[');
	mvwaddch(m_wins[1],3, 23, ']');
	mvwaddstr(m_wins[1],3, 24,"(???%)");
	mvwaddstr(m_wins[1],3, 30,"(0:00:00)");
	mvwhline(m_wins[1],4, 1, ':', STD_WID-2);
	mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");

	CreateListMenu("music.lst");

	CreatePluginMenu();

	CreateModeMenu();

	mvwprintw(m_wins[2], m_nLINES-STD_HEG-4, 1,
			"[%s]==[LIST]==[LRCD]==[PLUS]========",
			m_strMode[m_nCurMode].c_str());

	m_LrcMgr.SetHight(m_nLINES-8);

	//panels
	m_panels[0] = new_panel(m_wins[0]);
	m_panels[1] = new_panel(m_wins[1]);
	m_panels[2] = new_panel(m_wins[2]);
	m_panels[3] = new_panel(m_wins[3]);

	update_panels();
	doupdate();

	//	move(0, 0);

	keypad(stdscr, TRUE);
	mousemask(ALL_MOUSE_EVENTS, NULL);

	pthread_t p_thread;
	pthread_create(&p_thread, 0, LoopProc, this);
}

void CUIMgr::ReSize(bool bForce)
{
	if (m_nLINES != LINES || m_nCOLS != COLS || bForce)
	{
		m_nLINES = LINES;
		m_nCOLS = COLS;
		WINDOW *wins[4];
		wins[0] = newwin(m_nLINES, m_nCOLS, 0, 0);
		wins[1] = newwin(STD_HEG, STD_WID, 1, 1);
		wins[2] = newwin(m_nLINES-STD_HEG-2, STD_WID, STD_HEG+1, 1);
		wins[3] = newwin(m_nLINES-2, m_nCOLS-STD_WID-2, 1, STD_WID+1);
		for (int i = 0; i < 4; ++i)
		{
			if (isText)
			{
				wborder(wins[i],'|','|','-','-','+','+','+','+');
			}
			else
			{
				box(wins[i], 0, 0);				
			}
		}
		mvwhline(wins[1],3, 2, '-', 21);
		mvwaddch(wins[1],3, 1, '[');
		mvwaddch(wins[1],3, 23, ']');
		mvwaddstr(wins[1],3, 24,"(???%)");
		mvwaddstr(wins[1],3, 30,"(0:00:00)");
		mvwhline(wins[1],4, 1, ':', STD_WID-2);
		mvwaddstr(wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");

		if (m_menu)
		{
			unpost_menu(m_menu);
			set_menu_win(m_menu, wins[2]);
			set_menu_sub(m_menu, derwin(wins[2], m_nLINES-STD_HEG-7, 
			STD_WID-2, 2, 1));
			set_menu_format(m_menu, m_nLINES-STD_HEG-7, 1);
			if (!m_bPluginLock)
			{
				post_menu(m_menu);
			}
		}

		if (m_plusmenu)
		{
			unpost_menu(m_plusmenu);
			set_menu_win(m_plusmenu, wins[2]);
			set_menu_sub(m_plusmenu, derwin(wins[2], m_nLINES-STD_HEG-7,
					STD_WID-2, 2, 1));
			set_menu_format(m_plusmenu, m_nLINES-STD_HEG-7, 1);
			if (m_bPluginLock)
			{
				post_menu(m_plusmenu);
			}
		}

		if (m_modemenu)
		{
			unpost_menu(m_modemenu);
			set_menu_win(m_modemenu, wins[2]);
			set_menu_sub(m_modemenu, derwin(wins[2], 1, STD_WID-2, m_nLINES
					-STD_HEG-2-3, 1));
			if (m_bModeLock)
			{
				post_menu(m_modemenu);
			}
		}

		mvwprintw(wins[2], m_nLINES-STD_HEG-4, 1,
				"[%s]==[LIST]==[LRCD]==[PLUS]========",
				m_strMode[m_nCurMode].c_str());

		m_LrcMgr.SetHight(m_nLINES-8);

		for (int i = 0; i < 4; ++i)
		{
			replace_panel(m_panels[i], wins[i]);
			delwin(m_wins[i]);
			m_wins[i]=wins[i];
		}

		update_panels();
		doupdate();

		mvwhline(m_wins[1],1, 1, ' ', STD_WID-2);
		mvwprintw(m_wins[1], 1, 1, m_strName.c_str());
		mvwhline(m_wins[1],2, 1, ' ', STD_WID-2);
		mvwprintw(m_wins[1], 2, 1, m_strInfo.c_str());
		wrefresh(m_wins[1]);

		if (m_bPluginLock)
		{
			int nIndex = item_index(current_item(m_plusmenu));
			PrintPlugin(m_PluginMgr.GetDesc(nIndex));
		}
		else
		{
			RefreshLrc(m_fTPos, true);
		}
	}
}

void* CUIMgr::LoopProc(void* pData)
{
	CUIMgr* pMgr = (CUIMgr*)pData;
	if (pMgr)
	{
		pMgr->InputLoop();
	}
	return 0;
}

void CUIMgr::InputLoop()
{
	m_bRun = true;
	int c;
	while ((c = getch()) != 'q')
	{
		int nIndex = m_PluginMgr.OnHotKey(c);
		if (nIndex != -1)
		{
			m_bPainLock = true;
			echo();
			curs_set(2);
			def_prog_mode();
			endwin();
			m_PluginMgr.Run(nIndex);
			reset_prog_mode();
			noecho();
			curs_set(0);
			refresh();
			m_bPainLock = false;
			continue;
		}
		switch (c)
		{
		case 'f':
		{
			ReSize(true);
		}
			break;
		case 'g':
		{
			Command(eUCPlus);
		}
			break;
		case ',':
		{
			Command(eUCPrev);
		}
			break;
		case '.':
		{
			Command(eUCNext);
		}
			break;
		case 'p':
		{
			Command(eUCPlay);
		}
			break;
		case 's':
		{
			Command(eUCStop);
		}
			break;
		case 'm':
		{
			Command(eUCMute);
		}
			break;
		case 'd':
		{
			Command(eUCMode);
		}
			break;
		case 'l':
		{
			Command(eUCList);
		}
			break;
		case 'r':
		{
			Command(eUCLrcd);
		}
			break;
		case 10://ENTER
		{
			if (m_menu)
			{
				int nIndex = item_index(current_item(m_menu));
				m_nCurSel = nIndex;
				SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
			}
		}
			break;
		case KEY_MOUSE:
		{
			MEVENT event;
			if (getmouse(&event) == OK)
			{
				if ((event.bstate & BUTTON1_CLICKED) || (event.bstate
						& BUTTON1_DOUBLE_CLICKED))
				{
					MouseHint(&event);
				}
				else if ((event.bstate & BUTTON2_PRESSED))
				{
					if (m_menu)
					{
						menu_driver(m_menu, REQ_SCR_DPAGE);
						wrefresh(m_wins[2]);
					}
				}
				else if (event.bstate & BUTTON4_PRESSED)
				{
					if (m_menu)
					{
						menu_driver(m_menu, REQ_SCR_UPAGE);
						wrefresh(m_wins[2]);
					}
				}
			}
		}
			break;
		case KEY_DOWN:
		{
			if (m_menu)
			{
				menu_driver(m_menu, (item_index(current_item(m_menu))
						>= item_count(m_menu)-1) ? REQ_FIRST_ITEM
						: REQ_DOWN_ITEM);
				wrefresh(m_wins[2]);
			}
		}
			break;
		case KEY_UP:
		{
			if (m_menu)
			{
				menu_driver(m_menu, (item_index(current_item(m_menu)) <= 0) ? 
				REQ_LAST_ITEM : REQ_UP_ITEM);
				wrefresh(m_wins[2]);
			}
		}
			break;
		case KEY_NPAGE:
		{
			if (m_menu)
			{
				menu_driver(m_menu, REQ_SCR_DPAGE);
				wrefresh(m_wins[2]);
			}
		}
			break;
		case KEY_PPAGE:
		{
			if (m_menu)
			{
				menu_driver(m_menu, REQ_SCR_UPAGE);
				wrefresh(m_wins[2]);
			}
		}
			break;
		default:
		{
			//			if (c >= 0 && c <= 255) {
			//				SendMgrMsg(MSG_CHAR,0,(LPARAM)&c);
			//			}
		}
			break;
		}
		//		move(0,0);
		doupdate();
	}
	m_bRun = false;
}

void CUIMgr::PaintLoop()
{
	int nCnt = 0;
	while (m_bRun)
	{
		if (!m_bPainLock)
		{
			nCnt ++;
			if (nCnt % 5) //for normal.
			{
				ReSize();
			}
			if (m_bPlaying) //playing now ...
			{
				m_fTPos += 0.2f;
				if (nCnt % 10) //for normal.
				{
					mvwprintw(m_wins[1], 3, 30, "(%s)", Float2Time(m_fTPos).c_str());
					wrefresh(m_wins[1]);
					RefreshLrc(m_fTPos);
				}
				if (nCnt % 10 == 0)//2 sec for ping ...
				{
					if (!CheckAutoNext())
					{
						if (m_fTLen > 0.0f)
						{
							int nPer = (int)((m_fTPos/m_fTLen)
									*(float)(STD_WID-2));
							mvwhline(m_wins[1],4, 1, ':', STD_WID-2);
							mvwhline(m_wins[1],4, 1, '>', nPer);
						}
						m_tSTime = time(NULL);
						SendMgrMsg(MSG_TPOS,0,0);
					}
				}
			}
			if (nCnt >= 100)
			{
				nCnt = 0;
			}
			usleep(200*1000);
		}

	}
}

void CUIMgr::MouseHint(MEVENT* event)
{
	if (!event)
	{
		return;
	}
	int x = event->x;
	int y = event->y;
	if (y >= STD_HEG+3 && y <= m_nLINES-5)
	{
		if (x >= 4 && x <= STD_WID-2)
		{
			int nOffset = y - (STD_HEG+3);
			int nIndex = top_row(m_menu) + nOffset;
			MenuToSel(nIndex);
			wrefresh(m_wins[2]);
			if (event->bstate & BUTTON1_DOUBLE_CLICKED)
			{
				m_nCurSel = nIndex;
				SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
			}
		}
	}
	if ((event->bstate & BUTTON1_CLICKED)==0)
	{
		return;
	}
	if (y == m_nLINES-3)
	{
		if (x >= 3 && x<= 7) //mode
		{
			Command(eUCMode);
		}
		else if (x>=11 && x<=15) //list
		{
			Command(eUCList);
		}
		else if (x>= 19 && x<=23) //lrc dir
		{
			Command(eUCLrcd);
		}
		else if (x>= 27 && x<=31) //plugin
		{
			Command(eUCPlus);
		}
	}
	else if (y == 4)
	{
		if (x >= 3 && x<=23) //volum
		{
			int nPercent = (x-3)*5;
			mvwhline(m_wins[1],3, 2, '-', 21);
			mvwaddch(m_wins[1],3, x-1, '$');
			mvwprintw(m_wins[1], 3, 24, "(%3d%)", nPercent);
			wrefresh(m_wins[1]);
			SendMgrMsg(MSG_VOLUM,0,(LPARAM)nPercent);
		}
	}
	else if (y == 5)
	{
		if (x >= 2 && x <= STD_WID + 2) //pos
		{
			int nPercent = (x-2)*100/STD_WID;
			SendMgrMsg(MSG_SETPOS,0,(LPARAM)nPercent);
		}
	}
	else if (y== 6)
	{
		if (x >= 3 && x <= 7) //prev
		{
			Command(eUCPrev);
		}
		else if (x>=11 && x<=15) //play
		{
			Command(eUCPlay);
		}
		else if (x>= 19 && x<=23) //stop
		{
			Command(eUCStop);
		}
		else if (x>= 27 && x<=31) //next
		{
			Command(eUCNext);
		}
		else if (x>= 35 && x<=39) //mute
		{
			Command(eUCMute);
		}
	}
}

void CUIMgr::Command(int nCmd)
{
	switch (nCmd)
	{
	case eUCPlus:
	{
		m_bPluginLock = true;
		unpost_menu(m_menu);
		post_menu(m_plusmenu);
		wrefresh(m_wins[2]);
		int nIndex = 0;
		nIndex = item_index(current_item(m_plusmenu));
		PrintPlugin(m_PluginMgr.GetDesc(nIndex));
		int ch;
		bool bOK = false;
		do
		{
			ch = getch();
			switch (ch)
			{
			case 'q':
			case 10://enter
			{
				bOK = true;
			}
				break;
			case KEY_UP:
			{
				menu_driver(m_plusmenu, REQ_UP_ITEM);
				wrefresh(m_wins[2]);
				nIndex = item_index(current_item(m_plusmenu));
				PrintPlugin(m_PluginMgr.GetDesc(nIndex));
			}
				break;
			case KEY_DOWN:
			{
				menu_driver(m_plusmenu, REQ_DOWN_ITEM);
				wrefresh(m_wins[2]);
				nIndex = item_index(current_item(m_plusmenu));
				PrintPlugin(m_PluginMgr.GetDesc(nIndex));
			}
				break;
			default:
				break;
			}
		} while (!bOK);
		if (10 == ch)
		{
			nIndex = item_index(current_item(m_plusmenu));
			m_bPainLock = true;
			echo();
			curs_set(2);
			def_prog_mode();
			endwin();
			m_PluginMgr.Run(nIndex);
			reset_prog_mode();
			noecho();
			curs_set(0);
			refresh();
			m_bPainLock = false;
		}
		unpost_menu(m_plusmenu);
		ClearLrc();
		post_menu(m_menu);
		wrefresh(m_wins[2]);
		m_bPluginLock = false;
		RefreshLrc(m_fTPos, true);
	}
		break;
	case eUCMute:
	{
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [mute]");
		wrefresh(m_wins[1]);
		SendMgrMsg(MSG_MUTE,0,0);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
		break;
	case eUCMode:
	{
		m_bModeLock = true;
		post_menu(m_modemenu);
		wrefresh(m_wins[2]);
		int ch;
		bool bOK = false;
		do
		{
			ch = getch();
			switch (ch)
			{
			case 'q':
			case 10://enter
			{
				bOK = true;
			}
				break;
			case KEY_RIGHT:
			{
				menu_driver(m_modemenu, REQ_RIGHT_ITEM);
				wrefresh(m_wins[2]);
			}
				break;
			case KEY_LEFT:
			{
				menu_driver(m_modemenu, REQ_LEFT_ITEM);
				wrefresh(m_wins[2]);
			}
				break;
			default:
				break;
			}
		} while (!bOK);
		if (10==ch)
		{
			m_nCurMode = item_index(current_item(m_modemenu));
			mvwprintw(m_wins[2], m_nLINES-STD_HEG-4, 1, "[%s]",
					m_strMode[m_nCurMode].c_str());
		}
		unpost_menu(m_modemenu);
		wrefresh(m_wins[2]);
		m_bModeLock = false;
	}
		break;
	case eUCPrev:
	{
		int nSize = m_vctMenu.size();
		if (nSize <= 0)
		{
			break;
		}
		mvwaddstr(m_wins[1], 5, 1, "[|<  ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
		MenuToSel(m_nCurSel);
		m_nCurSel --;
		if (m_nCurSel < 0)
		{
			m_nCurSel = nSize-1;
			menu_driver(m_menu, REQ_LAST_ITEM);
		}
		else
		{
			menu_driver(m_menu, REQ_UP_ITEM);
		}
		SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		wrefresh(m_wins[2]);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);

	}
		break;
	case eUCNext:
	{
		int nSize = m_vctMenu.size();
		if (nSize <= 0)
		{
			break;
		}
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [  >|]  [MUTE]");
		wrefresh(m_wins[1]);
		MenuToSel(m_nCurSel);
		m_nCurSel ++;
		if (m_nCurSel >= nSize)
		{
			m_nCurSel = 0;
			menu_driver(m_menu, REQ_FIRST_ITEM);
		}
		else
		{
			menu_driver(m_menu, REQ_DOWN_ITEM);
		}
		SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		wrefresh(m_wins[2]);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
		break;
	case eUCPlay:
	{
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [paus]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
		SendMgrMsg(MSG_PLAY,0,0);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
		break;
	case eUCStop:
	{
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [stop]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
		SendMgrMsg(MSG_STOP,0,0);
		usleep(200*1000);
		mvwaddstr(m_wins[1], 5, 1, "[ |< ]  [PAUS]  [STOP]  [ >| ]  [MUTE]");
		wrefresh(m_wins[1]);
	}
		break;
	case eUCList:
	{
		m_bPainLock = true;

		mvwaddstr(m_wins[2], m_nLINES-STD_HEG-4-1, 1, "file:");
		wrefresh(m_wins[2]);

		echo();
		curs_set(2);
		char str[80];
		mvwgetstr(m_wins[2],m_nLINES-STD_HEG-4-1, 7,str);
		if (strlen(str) > 0)
		{
			CreateListMenu(str);
		}
		curs_set(0);
		noecho();

		mvwhline(m_wins[2],m_nLINES-STD_HEG-4-1, 1, ' ', STD_WID-2);
		wrefresh(m_wins[2]);

		m_bPainLock = false;
	}
		break;
	case eUCLrcd:
	{
		m_bPainLock = true;

		mvwaddstr(m_wins[2], m_nLINES-STD_HEG-4-1, 1, "dir:");
		wrefresh(m_wins[2]);

		echo();
		curs_set(2);
		char str[80];
		mvwgetstr(m_wins[2],m_nLINES-STD_HEG-4-1, 6,str);
		if (strlen(str) > 0)
		{
			m_strLrcDir = str;
			if (m_strLrcDir.rfind('/')!=m_strLrcDir.length()-1)
			{
				m_strLrcDir += "/";
			}
		}
		curs_set(0);
		noecho();

		mvwhline(m_wins[2],m_nLINES-STD_HEG-4-1, 1, ' ', STD_WID-2);
		wrefresh(m_wins[2]);

		m_bPainLock = false;
	}
		break;
	default:
		break;
	}
}

void CUIMgr::CreatePluginMenu()
{
	m_PluginMgr.Load();
	int nSize = m_PluginMgr.GetSize();
	m_plusitems = (ITEM **)calloc(nSize+1, sizeof(ITEM *));
	for (int i = 0; i < nSize; ++i)
	{
		m_plusitems[i] = new_item(m_PluginMgr.GetName(i), NULL);
	}
	m_plusitems[nSize] = new_item(NULL, NULL);

	m_plusmenu = new_menu((ITEM **)m_plusitems);

	set_menu_win(m_plusmenu, m_wins[2]);
	set_menu_sub(m_plusmenu, derwin(m_wins[2], m_nLINES-STD_HEG-7, STD_WID-2,
			2, 1));
	set_menu_format(m_plusmenu, m_nLINES-STD_HEG-7, 1);

	set_menu_mark(m_plusmenu, " > ");
}

void CUIMgr::CreateModeMenu()
{
	m_nCurMode = 3;
	m_strMode[0] = "SING";
	m_strMode[1] = "SILP";
	m_strMode[2] = "MULT";
	m_strMode[3] = "MULP";
	m_strMode[4] = "RAND";
	m_modeitems = (ITEM **)calloc(6, sizeof(ITEM *));
	for (int i = 0; i < 5; ++i)
	{
		m_modeitems[i] = new_item(m_strMode[i].c_str(), NULL);
	}
	m_modeitems[5] = new_item(NULL, NULL);

	m_modemenu = new_menu((ITEM **)m_modeitems);

	set_menu_win(m_modemenu, m_wins[2]);
	set_menu_sub(m_modemenu, derwin(m_wins[2], 1, STD_WID-2, m_nLINES-STD_HEG-2
			-3, 1));
	set_menu_format(m_modemenu, 1, 5);

	set_menu_mark(m_modemenu, ">");
}

void CUIMgr::CreateListMenu(const char* lpszFile)
{
	DestroyListMenu();
	wrefresh(m_wins[2]);
	m_nCurSel = 0;
	std::ifstream infile(lpszFile);
	std::string strTmp, strMenu;
	size_t nRPos = 0;
	char buffGBK[2048];
	while (getline(infile, strTmp))
	{

		if ((nRPos =strTmp.rfind('/')) != std::string::npos)
		{
			strMenu = strTmp.substr(nRPos+1, strTmp.length()-nRPos-1);
		}
		else
		{
			strMenu = strTmp;
		}
		MENUITEM item;
		if (!isGBK)
		{
			//do nothing ...
		}
		else
		{
			memset(buffGBK, 0, sizeof(buffGBK));
			u2g(strMenu.c_str(),buffGBK);
			strMenu = buffGBK;
		}
		item.strName = strMenu;
		item.strPath = strTmp;
		m_vctMenu.push_back(item);
	}
	int nSize = m_vctMenu.size();
	if (nSize <= 0)
	{
		return;
	}
	m_items = (ITEM **)calloc(nSize+1, sizeof(ITEM *));

	for (int i = 0; i < nSize; ++i)
	{
		m_items[i] = new_item(m_vctMenu[i].strName.c_str(), NULL);
	}
	m_items[nSize] = new_item(NULL, NULL);

	m_menu = new_menu((ITEM **)m_items);

	set_menu_win(m_menu, m_wins[2]);
	set_menu_sub(m_menu, derwin(m_wins[2], m_nLINES-STD_HEG-7, STD_WID-2, 2, 1));
	set_menu_format(m_menu, m_nLINES-STD_HEG-7, 1);

	set_menu_mark(m_menu, " > ");

	post_menu(m_menu);
	wrefresh(m_wins[2]);
}

void CUIMgr::DestroyListMenu()
{
	if (m_menu)
	{
		unpost_menu(m_menu);
		free_menu(m_menu);
		int nSize = m_vctMenu.size();
		for (int i = 0; i < nSize; ++i)
		{
			free_item(m_items[i]);
		}
		free(m_items);
		m_vctMenu.clear();
		m_menu = NULL;
	}
}

void CUIMgr::MenuToSel(int nSel)
{
	if (!m_menu)
	{
		return;
	}
	int nIndex = item_index(current_item(m_menu));
	if (nIndex > nSel)
	{
		while (nIndex != nSel)
		{
			menu_driver(m_menu, REQ_UP_ITEM);
			nIndex --;
		}
	}
	else if (nIndex < nSel)
	{
		while (nIndex != nSel)
		{
			menu_driver(m_menu, REQ_DOWN_ITEM);
			nIndex ++;
		}
	}
}

std::string CUIMgr::Float2Time(float fTotalSec)
{
	int nTotalSec = (int)fTotalSec;
	int nSec = nTotalSec % 60;
	int nMin = (nTotalSec - nSec)/60;
	int nTmp;
	int nHour = 0;
	if (nMin >= 60)
	{
		nTmp = nMin % 60;
		nHour = (nMin - nTmp) / 60;
		nMin = nTmp;
	}
	char szTmp[32];
	sprintf(szTmp, "%.1d:%.2d:%.2d", nHour, nMin, nSec);
	std::string s(szTmp);
	return s;
}

bool CUIMgr::CheckAutoNext()
{
	bool bRet = false;
	double fSec = difftime(m_tRTime, m_tSTime);
	if (fSec > 2.0f || fSec < -2.0f)
	{
		bRet = true;
		switch (m_nCurMode)
		{
		case 0:
		{
			m_bPlaying = false;
		}
			break;
		case 1:
		{
			SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		}
			break;
		case 2:
		{
			int nSize = m_vctMenu.size();
			if (m_nCurSel == nSize - 1)
			{
				m_bPlaying = false;
			}
			else
			{
				Command(eUCNext);
			}
		}
			break;
		case 3:
		{
			Command(eUCNext);
		}
			break;
		case 4:
		{
			int nSize = m_vctMenu.size();
			srand(time(NULL));
			int nTmp;
			if (nSize>1)
			{
				do
				{
					nTmp = rand() % m_vctMenu.size();
				} while (nTmp == m_nCurSel);
			}
			else
			{
				nTmp = 0;
			}
			m_nCurSel = nTmp;
			MenuToSel(m_nCurSel);
			wrefresh(m_wins[2]);
			SendMgrMsg(MSG_FILE,0,(LPARAM)m_vctMenu[m_nCurSel].strPath.c_str());
		}
			break;
		default:
			break;
		}
		m_tRTime = m_tSTime;
	}
	return bRet;
}

void CUIMgr::PrintLrc(const char* lpszStr)
{
	std::string str = lpszStr;
	size_t nPos = 0;
	size_t nStart = 0;
	size_t nLen = 0;
	int nLine = 3;
	size_t nPrintWid = (m_nCOLS-STD_WID-4)/2;
	int nOffset = 0;
	int nMid = (m_nLINES-8)/2; // in fact the index. base 0.
	int nCnt1 = 0;
	int nCnt2 = 0;

	int nSize = strlen(lpszStr);
	wchar_t* wcs = new wchar_t[nSize];
	memset(wcs, 0, nSize*sizeof(wchar_t));
	mbstowcs(wcs, lpszStr, nSize-1);
	std::wstring wstr(wcs);
	delete[] wcs;
#ifndef _USE_CURSESW
	char* cs = new char[nSize*2];
#endif
	while ((nPos=wstr.find(L'\n', nStart))!=std::wstring::npos && nLine <= m_nLINES-6)
	{
		nCnt1++;
		nLen = nPos - nStart;
		if (nLen > nPrintWid)
		{
			nLen = nPrintWid;
			wstr.insert(nStart + nLen, L"\n");
			nPos = nStart + nLen;
			nCnt2++;
			if (nCnt1-nCnt2<nMid)
			{
				nOffset ++;
			}
		}
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		if (nLen > 0)
		{
#ifndef _USE_CURSESW
			memset(cs,0,nSize*2);
			wcstombs(cs,wstr.substr(nStart,nLen).c_str(),nSize*2);
			mvwaddstr(m_wins[3], nLine, 1, cs);
#else
			mvwaddwstr(m_wins[3], nLine, 1, wstr.substr(nStart,nLen).c_str());
#endif
		}
		nLine ++;
		nStart = nPos+1;
	}
#ifndef _USE_CURSESW
	delete []cs;
#endif
	mvwchgat(m_wins[3],nMid+3+nOffset,1,m_nCOLS-STD_WID-4, A_REVERSE, 0, NULL);
	wrefresh(m_wins[3]);
}

void CUIMgr::PrintPlugin(const char* lpszStr)
{
	ClearLrc();
	std::string str = lpszStr;
	size_t nPos = 0;
	size_t nStart = 0;
	size_t nLen = 0;
	int nLine = 3;
	size_t nPrintWid = (m_nCOLS-STD_WID-4)/2;

	int nSize = strlen(lpszStr);
	wchar_t* wcs = new wchar_t[nSize];
	memset(wcs, 0, nSize*sizeof(wchar_t));
	mbstowcs(wcs, lpszStr, nSize-1);
	std::wstring wstr(wcs);
	delete[] wcs;
#ifndef _USE_CURSESW
	char* cs = new char[nSize*2];
#endif
	while ((nPos=wstr.find(L'\n', nStart))!=std::wstring::npos && nLine <= m_nLINES-6)
	{
		nLen = nPos - nStart;
		if (nLen > nPrintWid)
		{
			nLen = nPrintWid;
			wstr.insert(nStart + nLen, L"\n");
			nPos = nStart + nLen;
		}
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		if (nLen > 0)
		{
#ifndef _USE_CURSESW
			memset(cs,0,nSize*2);
			wcstombs(cs,wstr.substr(nStart,nLen).c_str(),nSize*2);
			mvwaddstr(m_wins[3], nLine, 1, cs);
#else
			mvwaddwstr(m_wins[3], nLine, 1, wstr.substr(nStart,nLen).c_str());
#endif
		}
		nLine ++;
		nStart = nPos+1;
	}
#ifndef _USE_CURSESW
	delete []cs;
#endif
	wrefresh(m_wins[3]);
}

int CUIMgr::SendMgrMsg(const char* lpszMsg, WPARAM wParam, LPARAM lParam)
{
	if (0 == strcmp(lpszMsg, MSG_FILE))
	{
		if (lParam)
		{
			ClearLrc();
			bool bRet = false;
			std::string str = GetLrcPath((const char*)lParam, 0);
			if (!str.empty())
			{
				m_nLastLrcIndex = -1;
				bRet = m_LrcMgr.LoadFile(str.c_str());
			}
			if (!bRet)
			{
				str = GetLrcPath((const char*)lParam, 1);
				if (!str.empty())
				{
					m_nLastLrcIndex = -1;
					bRet = m_LrcMgr.LoadFile(str.c_str());
				}
			}
			if (!bRet)
			{
				str = GetLrcPath((const char*)lParam, 2);
				if (!str.empty())
				{
					m_nLastLrcIndex = -1;
					bRet = m_LrcMgr.LoadFile(str.c_str());
				}
			}
		}
	}
	return CManager::SendMgrMsg(lpszMsg, wParam, lParam);
}

std::string CUIMgr::GetLrcPath(const char* lpszFile, int nType)
{
	//0:LrcDir 1:music path 2:Lrc dir
	std::string str;
	std::string strFileName;
	if (nType >= 0 || nType <= 2)
	{
		str = lpszFile;
		size_t nRPos;
		if ((nRPos =str.rfind('/')) != std::string::npos)
		{
			strFileName = str.substr(nRPos+1, str.length()-nRPos-1); //file name
		}
		str = "";
		if (0 == nType)
		{
			if (!m_strLrcDir.empty())
			{
				str = m_strLrcDir + strFileName; //path but no lrc.
			}
		}
		else if (1 == nType)
		{
			str = lpszFile;
		}
		else if (2 == nType)
		{
			str = "./Lrc/"+strFileName;
		}
		//chg to .lrc file.
		nRPos = str.rfind('.');
		if (nRPos != std::string::npos)
		{
			str = str.substr(0, nRPos);
			str += ".lrc";
		}
	}
	return str;
}

void CUIMgr::RefreshLrc(float fPos, bool bForce)
{
	if (m_LrcMgr.IsEmpty() || m_bPluginLock)
	{
		return;
	}
	int nIndex = m_LrcMgr.GetIndex(fPos);
	if (nIndex != m_nLastLrcIndex || bForce)
	{
		std::string str = m_LrcMgr.GetStr(nIndex);
		PrintLrc(str.c_str());
		m_nLastLrcIndex = nIndex;
	}
}

void CUIMgr::ClearLrc()
{
	int nLine = 3;
	while (nLine <= m_nLINES-6)
	{
		mvwhline(m_wins[3],nLine, 1, ' ', m_nCOLS-STD_WID-4);
		nLine ++;
	}
	wrefresh(m_wins[3]);
}

